@@ -128,6 +128,18 @@ public:
 
     void block__set_history(unsigned history) { return gr::block::set_history(history); }
 
+    void block__set_block_alias(std::string alias)
+    {
+        return gr::block::set_block_alias(alias);
+    }
+
+    std::string block__alias(void) const { return gr::block::alias(); }
+
+    void block__set_processor_affinity(std::vector<int> mask)
+    {
+        return gr::block::set_processor_affinity(mask);
+    }
+
     void block__set_fixed_rate(bool fixed_rate)
     {
         return gr::block::set_fixed_rate(fixed_rate);
@@ -310,7 +322,11 @@ protected:
 
     bool has_msg_handler(pmt::pmt_t which_port)
     {
-        return (d_msg_handlers_feval.find(which_port) != d_msg_handlers_feval.end());
+        if (d_msg_handlers_feval.find(which_port) != d_msg_handlers_feval.end()) {
+            return true;
+        } else {
+            return gr::basic_block::has_msg_handler(which_port);
+        }
     }
 
     void dispatch_msg(pmt::pmt_t which_port, pmt::pmt_t msg)
