@@ -24,28 +24,29 @@
 #define INCLUDED_GR_PREFS_H
 
 #include <gnuradio/api.h>
-#include <string>
-#include <map>
 #include <gnuradio/thread/thread.h>
+#include <map>
+#include <string>
 
 namespace gr {
 
-  typedef std::map< std::string, std::map<std::string, std::string> > config_map_t;
-  typedef std::map< std::string, std::map<std::string, std::string> >::iterator config_map_itr;
-  typedef std::map<std::string, std::string> config_map_elem_t;
-  typedef std::map<std::string, std::string>::iterator config_map_elem_itr;
+typedef std::map<std::string, std::map<std::string, std::string>> config_map_t;
+typedef std::map<std::string, std::map<std::string, std::string>>::iterator
+    config_map_itr;
+typedef std::map<std::string, std::string> config_map_elem_t;
+typedef std::map<std::string, std::string>::iterator config_map_elem_itr;
 
-  /*!
-   * \brief Base class for representing user preferences a la windows INI files.
-   * \ingroup misc
-   *
-   * The real implementation is in Python, and is accessable from C++
-   * via the magic of SWIG directors.
-   */
-  class GR_RUNTIME_API prefs
-  {
-  public:
-    static prefs *singleton();
+/*!
+ * \brief Base class for representing user preferences a la windows INI files.
+ * \ingroup misc
+ *
+ * The real implementation is in Python, and is accessible from C++
+ * via the magic of SWIG directors.
+ */
+class GR_RUNTIME_API prefs
+{
+public:
+    static prefs* singleton();
 
     /*!
      * \brief Creates an object to read preference files.
@@ -76,7 +77,7 @@ namespace gr {
      * key1 = value1
      * \endcode
      */
-    void add_config_file(const std::string &configfile);
+    void add_config_file(const std::string& configfile);
 
     /*!
      * \brief Returns the configuration options as a string.
@@ -94,84 +95,77 @@ namespace gr {
     /*!
      * \brief Does \p section exist?
      */
-    virtual bool has_section(const std::string &section);
+    virtual bool has_section(const std::string& section);
 
     /*!
      * \brief Does \p option exist?
      */
-    virtual bool has_option(const std::string &section,
-                            const std::string &option);
+    virtual bool has_option(const std::string& section, const std::string& option);
 
     /*!
      * \brief If option exists return associated value; else
      * default_val.
      */
-    virtual const std::string get_string(const std::string &section,
-                                         const std::string &option,
-                                         const std::string &default_val);
+    virtual const std::string get_string(const std::string& section,
+                                         const std::string& option,
+                                         const std::string& default_val);
 
     /*!
      * \brief Set or add a string \p option to \p section with value
      * \p val.
      */
-    virtual void set_string(const std::string &section,
-                            const std::string &option,
-                            const std::string &val);
+    virtual void set_string(const std::string& section,
+                            const std::string& option,
+                            const std::string& val);
 
     /*!
      * \brief If option exists and value can be converted to bool,
      * return it; else default_val.
      */
-    virtual bool get_bool(const std::string &section,
-                          const std::string &option,
-                          bool default_val);
+    virtual bool
+    get_bool(const std::string& section, const std::string& option, bool default_val);
 
     /*!
      * \brief Set or add a bool \p option to \p section with value \p val.
      */
-    virtual void set_bool(const std::string &section,
-                          const std::string &option,
-                          bool val);
+    virtual void
+    set_bool(const std::string& section, const std::string& option, bool val);
 
     /*!
      * \brief If option exists and value can be converted to long,
      * return it; else default_val.
      */
-    virtual long get_long(const std::string &section,
-                          const std::string &option,
-                          long default_val);
+    virtual long
+    get_long(const std::string& section, const std::string& option, long default_val);
 
     /*!
      * \brief Set or add a long \p option to \p section with value \p val.
      */
-    virtual void set_long(const std::string &section,
-                          const std::string &option,
-                          long val);
+    virtual void
+    set_long(const std::string& section, const std::string& option, long val);
 
     /*!
      * \brief If option exists and value can be converted to double,
      * return it; else default_val.
      */
-    virtual double get_double(const std::string &section,
-                              const std::string &option,
-                              double default_val);
+    virtual double
+    get_double(const std::string& section, const std::string& option, double default_val);
 
     /*!
      * \brief Set or add a double \p option to \p section with value \p val.
      */
-    virtual void set_double(const std::string &section,
-                            const std::string &option,
-                            double val);
+    virtual void
+    set_double(const std::string& section, const std::string& option, double val);
 
-  protected:
+protected:
     virtual std::vector<std::string> _sys_prefs_filenames();
-    virtual void _read_files(const std::vector<std::string> &filenames);
-    virtual char * option_to_env(std::string section, std::string option);
+    virtual void _read_files(const std::vector<std::string>& filenames);
+    virtual char* option_to_env(std::string section, std::string option);
 
-  private:
+private:
     gr::thread::mutex d_mutex;
     config_map_t d_config_map;
-  };
+};
 
 } /* namespace gr */
 
